#include <stdarg.h>
#include <syslog.h>
#include <string.h>
#include <stdio.h>

#include "mctd.h"

static bool enable_at_command_log = true;

void syslog_open(const char *name)
{
    openlog(name, LOG_CONS | LOG_PID, LOG_USER);
}

void syslog_close()
{
    closelog();
}

void log_write(int priority, const char *format, ...)
{
    va_list arg;

    va_start(arg, format);
    vsyslog(priority, format, arg);
    va_end(arg);
}

static char at_log_buf[MCTD_AT_BUF_SIZE + 16];

static void copy_and_replace_crlf(const char *str)
{
    int i;
    int j = 0;

    memset(at_log_buf, 0, MCTD_AT_BUF_SIZE + 16);
    for(i = 0; i < MCTD_AT_BUF_SIZE; i++) {
        switch(str[i]) {
        case '\0':
            at_log_buf[j] = '\0';
            return;
        case '\r':
            if (j > MCTD_AT_BUF_SIZE + 11)
                return;
            sprintf(&at_log_buf[j], "<CR>");
            j+=4;
            break;
        case '\n':
            if (j > MCTD_AT_BUF_SIZE + 11)
                return;
            sprintf(&at_log_buf[j], "<LF>");
            j+=4;
            break;
        default:
            at_log_buf[j] = str[i];
            j++;
            break;
        }

        if (j == MCTD_AT_BUF_SIZE + 14)
            return;
    }
}

void log_at_recv(const char *str)
{
    if (!enable_at_command_log)
        return;

    copy_and_replace_crlf(str);
    log_write(LOG_INFO, "<== %s", at_log_buf);
}

void log_at_send(const char *str)
{
    if (!enable_at_command_log)
        return;

    copy_and_replace_crlf(str);
    log_write(LOG_INFO, "==> %s", at_log_buf);
}

void set_output_at_command_log(bool enable)
{
    enable_at_command_log = enable;
}
